@@ -152,6 +152,7 @@ class _TransientSshConfigSchema(_TransientConfigSchema):
     """
 
     name = fields.Str(allow_none=False)
+    wait = fields.Bool(missing=False)
     ssh_command = fields.Str(allow_none=True)
     ssh_bin_name = fields.Str(missing="ssh", allow_none=True)
     ssh_timeout = fields.Int(missing=90, allow_none=True)
@@ -219,6 +220,7 @@ class _TransientRunConfigSchema(_TransientConfigSchema):
     ssh_console = fields.Bool(missing=False)
     ssh_with_serial = fields.Bool(missing=False)
     shared_folder = fields.List(fields.Str(), missing=[])
+    sftp_bin_name = fields.Str(missing="sftp-server", allow_none=True)
     no_virtio_scsi = fields.Bool(missing=False)
 
 
