@@ -9,16 +9,18 @@ import signal
 import socket
 import subprocess
 import sys
+import tempfile
 import threading
 import time
 
-from typing import Any, Dict, DefaultDict, List, Optional, Callable, Union
+from typing import Any, Dict, DefaultDict, List, Optional, Callable, Union, Sequence, cast
 
 from . import linux
 from . import utils
 
 _QMP_DELAY_BETWEEN = 0.2
-QMP_DEFAULT_SYNC_TIME = 5
+QMP_DEFAULT_SYNC_TIME = 10
+QMP_DEFAULT_TIMEOUT = 10
 QmpMessage = Dict[Any, Any]
 QmpCallback = Callable[[QmpMessage], None]
 
@@ -30,8 +32,8 @@ class QmpClient:
     event_callbacks: DefaultDict[str, List[QmpCallback]]
     current_id: int
 
-    def __init__(self, port: int) -> None:
-        self.port = port
+    def __init__(self, socket_path: str) -> None:
+        self.socket_path = socket_path
         self.id_callbacks = collections.defaultdict(list)
         self.event_callbacks = collections.defaultdict(list)
         self.current_id = 0
@@ -46,12 +48,29 @@ class QmpClient:
         self.file.write((json.dumps(msg) + "\r\n").encode("utf-8"))
         self.file.flush()
 
-    def connect(self, timeout: float) -> None:
-        logging.info(f"Connecting to QMP socket at 127.0.0.1:{self.port}")
+    def __recv_msg(self) -> Optional[QmpMessage]:
+        msg_json = self.file.readline()
+        if msg_json is None or msg_json == b"":
+            return None
+        return cast(QmpMessage, json.loads(msg_json))
+
+    def __connect_socket(self) -> socket.socket:
+        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+        sock.connect(self.socket_path)
+
+        # Now that we have bound to the socket, unlink the file so we don't
+        # leave a bunch of random files lying around
+        if os.path.exists(self.socket_path):
+            os.unlink(self.socket_path)
+
+        return sock
+
+    def connect(self, timeout: float = QMP_DEFAULT_TIMEOUT) -> None:
+        logging.info(f"Connecting to QMP socket at {self.socket_path}")
         start = time.time()
         while time.time() - start < timeout:
             try:
-                self.sock = socket.create_connection(("127.0.0.1", self.port))
+                self.sock = self.__connect_socket()
                 logging.debug("QMP connection established")
 
                 # Make a file object so we can readline. QMP messages will always
@@ -59,8 +78,9 @@ class QmpClient:
                 # cannot set the socket to be non-blocking if we do this.
                 self.file = self.sock.makefile("rwb")
 
-                # Go ahead and enable sending commands
-                self.__send_msg({"execute": "qmp_capabilities"})
+                # Read the server hello (and block until we get one)
+                server_hello = self.__recv_msg()
+                logging.debug(f"Received server hello: {server_hello}")
 
                 # Start and drop this reference. Because the thread is a daemon it will
                 # be killed when python dies
@@ -68,21 +88,23 @@ class QmpClient:
                 thread.daemon = True
                 thread.start()
 
+                # Go ahead and enable sending commands
+                self.send_sync({"execute": "qmp_capabilities"})
+
                 return
-            except ConnectionRefusedError:
-                logging.debug("QMP connection refused. Waiting")
+            except FileNotFoundError:
+                logging.debug("QMP unix socket not available. Waiting")
                 time.sleep(_QMP_DELAY_BETWEEN)
         raise ConnectionRefusedError(
-            f"Unable to connect to QMP socket at 127.0.0.1:{self.port}"
+            f"Unable to connect to QMP socket at {self.socket_path}"
         )
 
     def __start(self) -> None:
         while True:
-            msg_json = self.file.readline()
-            if msg_json is None or msg_json == b"":
+            msg = self.__recv_msg()
+            if msg is None:
                 logging.debug("QEMU closed QMP connection")
                 return
-            msg = json.loads(msg_json)
             logging.debug(f"Received QMP message: {msg}")
             if "id" in msg:
                 for callback in self.id_callbacks[msg["id"]]:
@@ -143,6 +165,7 @@ class QemuRunner:
     interactive: bool
     qmp_client: Optional[QmpClient]
     env: Optional[Dict[str, str]]
+    qmp_socket_path: Optional[str]
 
     # As far as I can tell, this _has_ to be quoted. Otherwise, it will
     # fail at runtime because I guess something is actually run here and
@@ -154,13 +177,13 @@ class QemuRunner:
         args: List[str],
         *,
         bin_name: Optional[str] = None,
-        qmp_port: Optional[int] = None,
         quiet: bool = False,
         interactive: bool = True,
         qmp_connectable: bool = False,
+        qmp_socket_path: Optional[str] = None,
         env: Optional[Dict[str, str]] = None,
+        pass_fds: Optional[Sequence[int]] = tuple(),
     ) -> None:
-        qmp_port = qmp_port or utils.allocate_random_port()
         self.bin_name = bin_name or self.__find_qemu_bin_name()
         self.quiet = quiet
         self.args = args
@@ -168,13 +191,19 @@ class QemuRunner:
         self.qmp_client = None
         self.interactive = interactive
         self.env = env
+        self.pass_fds = pass_fds
+
+        if qmp_socket_path is not None:
+            self.qmp_socket_path = qmp_socket_path
+        else:
+            self.qmp_socket_path = utils.generate_unix_socket_path()
 
         if qmp_connectable is True:
-            self.qmp_client = QmpClient(qmp_port)
-            self.args.extend(self.__default_qmp_args(qmp_port))
+            self.qmp_client = QmpClient(self.qmp_socket_path)
+            self.args.extend(self.__default_qmp_args(self.qmp_socket_path))
 
-    def __default_qmp_args(self, port: int) -> List[str]:
-        return ["-qmp", f"tcp:127.0.0.1:{port},server,nowait"]
+    def __default_qmp_args(self, socket_path: str) -> List[str]:
+        return ["-qmp", f"unix:{socket_path},server,nowait"]
 
     def __find_qemu_bin_name(self) -> str:
         return "qemu-system-x86_64"
@@ -201,6 +230,7 @@ class QemuRunner:
             # Automatically send SIGTERM to this process when the main Transient
             # process dies
             preexec_fn=lambda: linux.set_death_signal(signal.SIGTERM),
+            pass_fds=self.pass_fds,
         )
 
         return self.proc_handle
