@@ -1,4 +1,3 @@
-from . import checked_threading
 from . import configuration
 from . import editor
 from . import qemu
@@ -16,7 +15,6 @@ import glob
 import json
 import logging
 import os
-import pwd
 import signal
 import subprocess
 import tempfile
@@ -44,9 +42,6 @@ else:
     Environ = os._Environ
 
 
-MAX_CONCURRENT_SSHFS = 7
-
-
 @enum.unique
 class TransientVmState(enum.Enum):
     WAITING = (1,)
@@ -161,15 +156,16 @@ class TransientVm:
         if not vm_absolute_path.startswith("/"):
             raise RuntimeError(f"Absolute path for guest required: {vm_absolute_path}")
 
-        for vm_image in self.vm_images:
-            assert isinstance(vm_image, image.FrontendImageInfo)
-            assert vm_image.backend is not None
-            logging.info(
-                f"Copying from '{host_path}' to '{vm_image.backend.identifier}:{vm_absolute_path}'"
-            )
+        # For now copy only to the first "-image" specified.
+        vm_image = self.vm_images[0]
+        assert isinstance(vm_image, image.FrontendImageInfo)
+        assert vm_image.backend is not None
+        logging.info(
+            f"Copying from '{host_path}' to '{vm_image.backend.identifier}:{vm_absolute_path}'"
+        )
 
-            with editor.ImageEditor(self.config, vm_image.path) as edit:
-                edit.copy_in(host_path, vm_absolute_path)
+        with editor.ImageEditor(self.config, vm_image.path) as edit:
+            edit.copy_in(host_path, vm_absolute_path)
 
     def __needs_to_copy_out_files_after_running(self) -> bool:
         """Checks if at least one directory on the VM needs to be copied out
@@ -199,15 +195,16 @@ class TransientVm:
         if not vm_absolute_path.startswith("/"):
             raise RuntimeError(f"Absolute path for guest required: {vm_absolute_path}")
 
-        for vm_image in self.vm_images:
-            assert isinstance(vm_image, image.FrontendImageInfo)
-            assert vm_image.backend is not None
-            logging.info(
-                f"Copying from '{vm_image.backend.identifier}:{vm_absolute_path}' to '{host_path}'"
-            )
+        # For now copy only to the first "-image" specified.
+        vm_image = self.vm_images[0]
+        assert isinstance(vm_image, image.FrontendImageInfo)
+        assert vm_image.backend is not None
+        logging.info(
+            f"Copying from '{vm_image.backend.identifier}:{vm_absolute_path}' to '{host_path}'"
+        )
 
-            with editor.ImageEditor(self.config, vm_image.path) as edit:
-                edit.copy_out(vm_absolute_path, host_path)
+        with editor.ImageEditor(self.config, vm_image.path) as edit:
+            edit.copy_out(vm_absolute_path, host_path)
 
     def __qemu_added_args(self) -> List[str]:
         new_args = ["-name", self.name]
@@ -265,6 +262,7 @@ class TransientVm:
             port=self.set_ssh_port,
             user=self.config.ssh_user,
             ssh_bin_name=self.config.ssh_bin_name,
+            sftp_server_bin_name=self.config.sftp_server_bin_name,
         )
 
     def __connect_ssh(self) -> int:
@@ -275,9 +273,6 @@ class TransientVm:
         conn.wait()
         return conn.returncode
 
-    def __current_user(self) -> str:
-        return pwd.getpwuid(os.getuid()).pw_name
-
     def __qemu_sigchld_handler(self, sig: int, frame: Any) -> None:
         # We register this signal handler after the QEMU start, so these must not be None
         assert self.qemu_runner is not None
@@ -414,11 +409,7 @@ class TransientVm:
         if self.__needs_ssh():
             self.__prepare_ssh()
 
-        # Now that we know the ssh port (if any), we can write out the
-        # proc data to our tempfile
-        self.__prepare_proc_data()
-
-        sshfs_threads: List[checked_threading.Thread] = []
+        sshfs_threads = []
         for shared_spec in self.config.shared_folder:
             assert self.ssh_config is not None
             local, remote = shared_spec.split(":")
@@ -427,24 +418,21 @@ class TransientVm:
             # so make it absolute
             absolute_local_path = os.path.abspath(local)
             sshfs_kwargs = {
-                "connect_timeout": self.config.ssh_timeout,
+                "ssh_timeout": self.config.ssh_timeout,
                 "ssh_config": self.ssh_config,
                 "local_dir": absolute_local_path,
                 "remote_dir": remote,
-                "local_user": self.__current_user(),
             }
 
-            # Slamming the server with 20 connections at once is a good way to break things:
-            if len(sshfs_threads) > MAX_CONCURRENT_SSHFS:
-                sshfs_threads.pop(0).join()
-
-            sshfs_threads.append(
-                checked_threading.Thread(target=sshfs.do_sshfs_mount, kwargs=sshfs_kwargs)
-            )
+            sshfs_threads.append(sshfs.SshfsThread(**sshfs_kwargs))
             sshfs_threads[-1].start()
 
         for sshfs_thread in sshfs_threads:
-            sshfs_thread.join()
+            sshfs_thread.wait_for_mount(self.config.ssh_timeout)
+
+        # Now that we know the ssh port (if any) and we've set up shared folders,
+        # we can write out the proc data to our tempfile
+        self.__prepare_proc_data()
 
         if self.__needs_ssh_console():
             # Note that we always return the SSH exit code, even if the guest failed to
